<?php

/**
 * Fichero de funciones auxiliares para SocialSight
 * @version 0.2
 * @author Miguel Sempere Sanchez <miguel.sempere.sanchez@gmail.com>
 */


/**
* AAP_ID define el numero de identificacion de SocialSight dentro del conjunto de aplicaciones de Facebook
*/
define("APP_ID", '138377122886126');
/**
 * API_SECRET define el numero secreto de la aplicacion
 */
define("API_SECRET", '4e9d49f390b063700817e9209fb80e99');
/**
 * SCOPE define los permisos de la aplicacion sobre las caracteristicas de Facebook
 */
define("SCOPE","publish_stream,read_stream,user_events,user_hometown,friends_hometown,user_location,friends_location");
/**
 * REDIRECT_URL define la web a la que es redirigida la aplicacion tras realizar el login en la web de Facebook
 */
define("REDIRECT_URL",'http://localhost/principal.php');






class SocialSight{

	private $LOGIN_URL;
	private $INSTANCE;
	private static $instaciaSingletonSS;
	private $access_token;


	/**
	 * Constructor de la clase SocialSight de tipo Singleton
	 *
	 */
	function __construct(){
		$this->LOGIN_URL="https://www.facebook.com/dialog/oauth?client_id=".APP_ID."&redirect_uri=".REDIRECT_URL."&scope=".SCOPE;
		$this->INSTANCE = new Facebook(array('appId'  => APP_ID,'secret' => API_SECRET,'cookie' => true, ));
		$this->access_token=null;
	}

	/**
	 * Permite obtener la instacia del objeto Socialsight si este ya ha sido creado, o crearlo en caso contrario
	 *
	 * @return SocialSight instancia
	 */
	public static function getInstanciaSingleton(){
		if (  !self::$instaciaSingletonSS instanceof self){
			self::$instaciaSingletonSS = new self;
		}
		return self::$instaciaSingletonSS;
	}

	/**
	 * Obtenemos la url a traves de la que el usuario realizara el logout
	 *
	 * @return url format: http://localhost/socialsight/salir.php?FACEBOOK_TOKENS
	 */
	public function getExitUrl(){
		$tmp=urldecode(SocialSight::getInstanciaSingleton()->INSTANCE->getLogoutUrl(array('next' => 'http://localhost/socialsight/salir.php')));
		return $tmp;
	}

	/**
	 * Nos permite obtener la cadena de session creada al realizar el login
	 *
	 * @return string sesion
	 */
	public function getSession(){
		return SocialSight::getInstanciaSingleton()->INSTANCE->getSession();
	}


	/**
	 * Permite obtener la instacia del objeto Socialsight si este ya ha sido creado, o crearlo en caso contrario
	 *
	 * @return SocialSight instancia
	 */
	public function getInstancia(){
		return SocialSight::getInstanciaSingleton()->INSTANCE;
	}

	/**
	 * Obtenemos la url a traves de la que el usuario realizara el login
	 *
	 * @return url format: http://localhost/socialsight/index.php?FACEBOOK_TOKENS
	 */
	public function getLoginUrl(){
		return SocialSight::getInstanciaSingleton()->LOGIN_URL;
	}

	/**
	 * Nos permite obtener el token de acceso creado por Facebook que permite interactuar con el API
	 *
	 * @return string accesstoken
	 */
	public function getAccessToken(){
		$session=SocialSight::getInstanciaSingleton()->getSession();

		if($session){
			return $session['access_token'];
		}
		else{
			return null;
		}
	}
}



//FUNCIONES VISUALES

/**
 * Funcion que muestra el menu estatico en todas las paginas. Contiene Inicio, Perfil, Amigos, Salir...
 *
 * @return string con la formacion del div del menu
 */
function MostrarMenu(){

	$socialsight=SocialSight::getInstanciaSingleton();
	$facebook=$socialsight->getInstancia();
	$url=$socialsight->getExitUrl();
	$session = $socialsight->getSession();



	if(!$session)
		$url="/socialsight/salir.php";




	echo "<div>
			<ul><img src=\"/socialsight/mini_logo_socialsight.png\" alt=\"Mini Logotipo estatico de SocialSight, gato con bigotes\" ALIGN=RIGHT></ul>

			<ul>
				<li><a accesskey=\"I\" title=\"Pagina de inicio del usuario\" href=\"/socialsight/index.php\" hreflang=\"es\" shape=\"rect\">Inicio</a></li>
				<li><a accesskey=\"P\" title=\"Pagina de perfil del usuario\" href=\"/socialsight/perfil.php\" hreflang=\"es\" shape=\"rect\">Perfil</a></li>
				<li><a accesskey=\"A\" title=\"Pagina de amigos del usuario\" href=\"/socialsight/amigos.php?paginaDeAmigos=1\" hreflang=\"es\" shape=\"rect\">Amigos</a></li>
				<li><a accesskey=\"S\" title=\"Salir de SocialSight\" href=\"" .$url. "\" hreflang=\"es\" shape=\"rect\">Salir</a></li>
			</ul>
		</div>";
}


/**
 * Nos permite generar los numeros que dan acceso a las distintas paginas de los amigos de un usuario
 *
 * @param int $numPaginas numero de paginas que se generan con 10 amigos cada una
 * @return string que forma los enlaces de las paginas
 */
function paginadorAmigos($numPaginas){
	$cadena="Pagina ";
	for ( $i = 1 ; $i <= $numPaginas ; $i ++) {
		$cadena=$cadena. "<a href=\"/socialsight/amigos.php?paginaDeAmigos=".$i."\" shape=\"rect\"> ".$i.' </a>';
	}
	return $cadena;
}


/**
 * Introduce el tag HTML
 *
 * @return string con <HTML>
 */
function HTML(){
	//return "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\"> <HTML lang=\"es\">";
	return "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">
	<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"es\">";
}

/**
 * Introduce el codigo HTML para introducir la referencia al fichero de estilo de la plataforma
 *
 * @return string con la referencia absoluta al documento CSS
 */
function ImportarCSS(){
	echo "<link href=\"/socialsight/ssmain.css\" rel=\"stylesheet\" type=\"text/css\" media=\"screen\" />";
}


/**
 * Nos permite generar los tags de HEAD y TITLE para el documento actual
 *
 * @param string $titulo titulo que quedara en el interior del tag TITLE
 * @return string con el codigo HTML generado
 */
function MostrarCabecera($titulo){

	echo"<HEAD>";
	echo '<meta http-equiv="Content-Type" content="text/html; charset=utf-8"/>';
	echo "<TITLE>".$titulo."</TITLE>";
	ImportarCSS();
	echo"</HEAD>";
}


/**
 * Nos general el codigo HTML del formulario con el TextArea y el boton de introduccion de datos
 *
 * @param string $pagina con la pagina donde se redireccionara tras pulsar el boton
 * @param string $id del usuario sobre el que se enviaran los datos
 * @return string con el codigo HTML generando el FORM
 */
function MostrarInputForm($pagina,$id){

	echo "<form name=\"mensaje_publicar_en_muro\" class=\"mensaje_publicar_en_muro\" action="."\"$pagina".".php?id=".$id."\" method=\"post\">";
	echo "<textarea cols=\"50\" rows=\"4\" name=\"contenido_mensaje_deseado_poner_muro\" tabindex=\"1\"></textarea>";
	echo "<input type=\"submit\" name=\"boton_publicar_muro\" value=\"Publicar mensaje\" tabindex=\"2\">";
	echo "</form>";

}

//FUNCIONES MURO

/**
 * Funcion para publicar mensajes en el muro de los usuarios
 *
 * @param static INSTANCE $FacebookInstance con la instacia singleton del API de Facebook sobre la que estamos trabajando
 * @param string $TokenAcceso Token de acceso de la aplicacion a Facebook
 * @param string $Mensaje mensaje que se publicara en el muro del usuario deseado
 * @param int $quien usuario sobre el que escribiremos en su muro
 * @return 0 en caso correcto, -1 en caso de fallo
 */
function PublicarMuro($FacebookInstance, $TokenAcceso, $Mensaje, $quien){



	if($quien=="me"){
		$w='/me/feed';
	}
	else{
		$w='/'.$quien.'/feed';
	}


	$attachment =  array(
	'access_token' => $TokenAcceso,
	'message' => $Mensaje,
	'name' => "",
	'description' => ""
	);


	try{
		$FacebookInstance->api($w, 'POST', $attachment);
		return 0;
	}

	catch (FacebookApiException $e) {

		echo "Se ha producido un error, puede que se cerrase la sesion. Por favor salga y vuelva a conectarse de nuevo.";
		return -1;
	}
}


/**
 * Funcion para responder a mensajes sobre el muro
 *
 * @param static INSTANCE $FacebookInstance con la instacia singleton del API de Facebook sobre la que estamos trabajando
 * @param int $id referencia sobre el mensaje que queremos contestar
 * @param string $Mensaje mensaje que se empleara de respuesta
 * @return 0 en caso correcto, -1 en caso de fallo
 */
function ResponderMensaje($FacebookInstance,$id, $mensaje){

	$cadena="/".$id."/comments";

	try{
		$FacebookInstance->api($cadena,"POST",array("message"=>$mensaje));
		return 0;
	}
	catch (FacebookApiException $e) {
		echo "Se ha producido un error, puede que se cerrase la sesion. Por favor salga y vuelva a conectarse de nuevo.";
		return -1;
	}

}


//FUNCIONES OBTENCION DE DATOS

/**
 * Obtenemos un usuario determinado con todos sus datos a traves de su id
 *
 * @param static INSTANCE $FacebookInstance con la instacia singleton del API de Facebook sobre la que estamos trabajando
 * @param int $id referencia del usuario sobre el que obtener los datos
 * @return array con los datos del usuario
 */
function getCompleteUser($FacebookInstance,$id){
	$datos=$FacebookInstance->api('/'.$id);
	return $datos;
}


/**
 * Obtenemos los comentarios realizados sobre un post de un usuario en el muro
 *
 * @param static INSTANCE $FacebookInstance con la instacia singleton del API de Facebook sobre la que estamos trabajando
 * @param int $id referencia del post sobre el que consultar las respuestas
 * @return array con las respuestas
 */
function getComments($FacebookInstance, $idPost){
	try{
		$query="select likes,comments from stream where post_id='".$idPost."'";
		$respuesta = $FacebookInstance->api(array('method' => 'fql.query','query' =>$query,));
		return $respuesta;
	}
	catch(FacebookApiException $e){
		echo "Se ha producido un error, puede que se cerrase la sesion. Por favor salga y vuelva a conectarse de nuevo.";
		return -1;
	}
}

/**
 * Obtenemos el nombre del usuario a traves de una consulta FQL
 *
 * @param static INSTANCE $FacebookInstance con la instacia singleton del API de Facebook sobre la que estamos trabajando
 * @param int $id referencia del usuario sobre el que obtener el nombre
 * @return string con el nombre del usuario
 */
function getUserFQL($FacebookInstance,$id){
	try{
		$query="select name from user where uid='".$id."'";
		$respuesta = $FacebookInstance->api(array('method' => 'fql.query','query' =>$query,));
	}
	catch (FacebookApiException $e) {
		echo "Se ha producido un error, puede que se cerrase la sesion. Por favor salga y vuelva a conectarse de nuevo.";
	}

	return $respuesta;
}

/**
 * Obtenemos los datos de un comentario de muro a traves de una consulta FQL
 *
 * @param static INSTANCE $FacebookInstance con la instacia singleton del API de Facebook sobre la que estamos trabajando
 * @param int $id referencia del mensaje sobre el que obtener los datos
 * @return array con los datos
 */
function getStreamFQL($FacebookInstance,$id){
	try{
		$query="select source_id ,created_time,message,likes from stream where post_id='".$id."'";
		$respuesta = $FacebookInstance->api(array('method' => 'fql.query','query' =>$query,));
	}

	catch (FacebookApiException $e) {

		echo "Se ha producido un error, puede que se cerrase la sesion. Por favor salga y vuelva a conectarse de nuevo.";
	}

	return $respuesta;
}


/**
 * Obtenemos los eventos a los que pertenece el usuario
 *
 * @param static INSTANCE $FacebookInstance con la instacia singleton del API de Facebook sobre la que estamos trabajando
 * @param int $id referencia del usuario
 * @return array con los eventos
 */
function getEvents($FacebookInstance,$id){

	if($id=="me"){
		$events= $FacebookInstance->api('/me/events');
		return $events;
	}
	else{

	}
}

/**
 * Obtenemos la id del usuario actual
 *
 * @param static INSTANCE $FacebookInstance con la instacia singleton del API de Facebook sobre la que estamos trabajando
 * @return int de la identificacion del usuario
 */
function getMeId($FacebookInstance){
	return $FacebookInstance->getUser();
}

/**
 * Obtenemos los datos del usuario principal de la plataforma
 *
 * @param static INSTANCE $FacebookInstance con la instacia singleton del API de Facebook sobre la que estamos trabajando
 * @return array con los datos del usuario
 */
function getMe($FacebookInstance){

	$datos=$FacebookInstance->api('/me/');
	return $datos;
}

/**
 * Obtenemos los datos de un usuario a partir de su id, mediante CURL
 *
 * @param static INSTANCE $FacebookInstance con la instacia singleton del API de Facebook sobre la que estamos trabajando
 * @param int $id referencia del usuario sobre el que obtener datos
 * @return array de datos del usuario
 */
function getUser($access_token, $id){

	$url = 'https://graph.facebook.com/'.$id.'?access_token=' . $access_token;
	$ch = curl_init();
	curl_setopt($ch, CURLOPT_URL, $url);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
	curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
	$response = curl_exec($ch);
	curl_close($ch);

	//echo $response;

	$user = json_decode($response);

	return $user;
}

/**
 * Obtenemos la imagen de perfil de un usuario a traves de la id de este
 *
 * @param int $id referencia del usuario sobre el que obtener la imagen de perfil
 * @return url con el enlace a la imagen JPG
 */
function ObtenerURLImagenUsuarioID($id){
	return 'https://graph.facebook.com/'.$id.'/picture';
}

/**
 * Obtenemos el conjunto de amigos del usuario principal
 *
 * @param static INSTANCE $FacebookInstance con la instacia singleton del API de Facebook sobre la que estamos trabajando
 * @return array de amigos del usuario
 */
function ObtenerArrayAmigos($FacebookInstance){

	try{
		$amigos = $FacebookInstance->api('/me/friends');
	}
	catch (FacebookApiException $e) {

		echo "Se ha producido un error, puede que se cerrase la sesion. Por favor salga y vuelva a conectarse de nuevo.";
	}

	return $amigos;
}

/**
 * Obtenemos los mensajes del muro de un determinado usuario
 *
 * @param static INSTANCE $FacebookInstance con la instacia singleton del API de Facebook sobre la que estamos trabajando
 * @param int $id referencia del usuario sobre el que obtener su muro
 * @param int $numMensajes numeros de LIMITE de mensajes a obtener
 * @param string $camposMensaje atributos para incluir en la consulta de obtencion de feeds
 * @return array con los mensajes del muro del usuario
 */
function ObtenerFeed($id,$FacebookInstance, $numMensajes, $camposMensaje){

	if($id=="me"){
		$feed = $FacebookInstance->api('/me/feed?limit='.$numMensajes.'&fields='.$camposMensaje);
	}
	else{

		if($id!=null){

			$feed = $FacebookInstance->api('/'.$id.'/feed?limit='.$numMensajes.'&fields='.$camposMensaje);
		}
	}

	return $feed;

}


?>